local math, os = math, os
-- local timeutils = require "timeutils"
module (...)

--
local function rangemodulo (n, from, to)
	return from + (n-from)%(to-from)
end

local function rangepin (n, from, to)
	if n < from then return from end
	if n > to then return to end
	return n
end


-- the time as a fraction of day, [0,1)
local function fractday (date)
	return (3600*date.hour + 60*date.min + date.sec)/(24*3600)
end

-- the datetime as a fraction of year, [0, 2pi)  (radians)
local function fractyear (date)
	return (2*math.pi/365.25) * (date.yday + fractday (date))
end

local function solartime(date)
	return
end

-- solar declination of a date:
-- the latitude at which the sun is directly overhead, [-25.5, 23.5]
local function sundeclin (date)
	local g = fractyear (date)
	return math.rad(0.396372 - 22.91327*math.cos(g) + 4.02543*math.sin(g) - 0.387205*math.cos(2*g)+
					0.051967*math.sin(2*g) - 0.154527*math.cos(3*g) + 0.084798*math.sin(3*g))
end

-- time correction for solar angle
local function timecorrection (date)
	local g = fractyear (date)
	return 0.004297
		+ 0.107029*math.cos(g)
		-1.837877*math.sin(g)
		-0.837378*math.cos(2*g)
		-2.340475*math.sin(2*g)
end

-- solar hour angle (radians)
local function solarhourangle (date, place)
	local tc = timecorrection (date)
	return math.rad((fractday(date)*24 - 12)*15 + place.longitude + tc)
end

-- sun zenith angle
function sunzenithangle (date, place)
	local d = sundeclin (date)
	local sha = solarhourangle (date, place)
	local latrad = math.rad (place.latitude);
	return math.acos (math.sin (latrad)*math.sin(d)
					+ math.cos (latrad)*math.cos(d)*math.cos(sha))
end

-- solar elevation (degrees)
function sunelevation (date, place)
	return 90 - math.deg(sunzenithangle (date, place))
end

function sunazimuthangle (date, place)
	local d = sundeclin (date)
	local sza = sunzenithangle (date, place)
	local latrad = math.rad (place.latitude)
	return math.acos((math.sin(d)-math.sin(latrad)*math.cos(sza))/(math.cos(latrad)*math.sin(sza)))
end

